// © 2021 ETH Zurich, Mechanics and Materials Lab
//
// ae108 is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// ae108 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ae108. If not, see <https://www.gnu.org/licenses/>.

#pragma once

namespace ae108::meshing::cppgmsh {

/**
 * @brief Generate mesh. The mesh can be exported with extract_mesh.
 *
 * @param dimension The desired meshing dimension (e.g. volume (3) or surface
 * (2) mesh).
 * @param order The desired element order. See gmsh docs.
 * @param algorithm The desired meshing algorithm. See gmsh docs.
 * @note https://gitlab.onelab.info/gmsh/gmsh/-/blob/gmsh_4_8_4/api/gmsh.h#L585
 */
void generate_mesh(const int dimension = 3, const int order = 1,
                   const int algorithm = 6) noexcept;

} // namespace ae108::meshing::cppgmsh